//
//  AppTests.swift
//  DIKitPackageDescription
//
//  Created by Yosuke Ishikawa on 2017/09/16.
//

import XCTest
import DIGenKit
import DIKit

final class DetailViewController: Injectable {
    struct Dependency {
        let index: Int
        let apiClient: APIClient
    }

    init(dependency: Dependency) {}
}

final class ListViewController: Injectable {
    struct Dependency {
        let apiClient: APIClient
        let resolver: AppResolver
    }

    init(dependency: Dependency) {}
}


final class APIClient {}

protocol AppResolver: Resolver {
    func provideAppResolver() -> AppResolver
    func provideAPIClient() -> APIClient
}

final class AppTests: XCTestCase {
    func test() throws {
        let generator = try CodeGenerator(path: #file)
        let contents = try generator.generate().trimmingCharacters(in: .whitespacesAndNewlines)
        XCTAssertEqual(contents, """
            //
            //  Resolver.swift
            //  Generated by dikitgen.
            //

            import DIGenKit
            import DIKit
            import XCTest

            extension AppResolver {

                func resolveAPIClient() -> APIClient {
                    return provideAPIClient()
                }

                func resolveAppResolver() -> AppResolver {
                    return provideAppResolver()
                }

                func resolveDetailViewController(index: Int) -> DetailViewController {
                    let apiClient = resolveAPIClient()
                    return DetailViewController(dependency: .init(index: index, apiClient: apiClient))
                }

                func resolveListViewController() -> ListViewController {
                    let apiClient = resolveAPIClient()
                    let appResolver = resolveAppResolver()
                    return ListViewController(dependency: .init(apiClient: apiClient, resolver: appResolver))
                }

            }
            """)
    }
}
